@@ -16,14 +16,15 @@
 # pacman -S git patch make autoconf automake nasm libtool
 # Start the msys shell, add Qt and cmake to the path and start this script.
 #
-# export QTPREFIX=/c/Qt/5.12.8/mingw73_64
+# export QTPREFIX=/c/Qt/6.5.1/mingw_64
 # test -z "${PATH##$QTPREFIX*}" ||
 # PATH=$QTPREFIX/bin:$QTPREFIX/../../Tools/mingw730_64/bin:$QTPREFIX/../../Tools/mingw730_64/opt/bin:$PROGRAMFILES/CMake/bin:$PATH
 # ../kid3/build.sh
 #
 # You can also build a Windows version from Linux using the MinGW cross
 # compiler.
-# COMPILER=cross-mingw QTPREFIX=/path/to/Qt5.6.3-mingw/5.6.3/mingw49_32 ../kid3/build.sh
+# COMPILER=cross-mingw QTPREFIX=/path/to/Qt6.5.1-mingw64/6.5.1/mingw_64 \
+#   QTBINARYDIR=/path/to/Qt6.5.1-linux/6.5.1/gcc_64/bin ../kid3/build.sh
 #
 # For Mac:
 #
@@ -31,22 +32,28 @@
 # can be installed with Homebrew, for instance:
 # brew install cmake ninja autoconf automake libtool xz nasm docbook-xsl
 # Then call from a build directory
-# QTPREFIX=/path/to/Qt/5.9.7/clang_64 ../kid3/build.sh
+# QTPREFIX=/path/to/Qt/6.5.1/clang_64 ../kid3/build.sh
 #
 # You can also build a macOS version from Linux using the osxcross toolchain.
-# COMPILER=cross-macos QTPREFIX=/path/to/Qt5.9.7-mac/5.9.7/clang_64 ../kid3/build.sh
+# COMPILER=cross-macos QTPREFIX=/path/to/Qt6.5.1-mac/6.5.1/macos ../kid3/build.sh
 # or
-# COMPILER=cross-macos QTPREFIX=/path/to/Qt5.9.7-mac/5.9.7/clang_64 QTBINARYDIR=/path/to/Qt5.9.7-linux/5.9.7/gcc_64/bin ../kid3/build.sh
+# COMPILER=cross-macos QTPREFIX=/path/to/Qt6.5.1-mac/6.5.1/macos \
+#   QTBINARYDIR=/path/to/Qt6.5.1-linux/6.5.1/gcc_64/bin ../kid3/build.sh
 #
 # For Android:
 #
-# Install Qt and a compatible Android SDK and NDK, for example Qt 5.9.7, NDK 10e or Qt 5.12.2, NDK 19c.
-# COMPILER=cross-android QTPREFIX=/path/to/Qt/5.9.7/android_armv7 ANDROID_SDK_ROOT=/path/to/sdk ANDROID_NDK_ROOT=/path/to/ndk-bundle ../build.sh
+# Install Qt and a compatible Android SDK and NDK, for example Qt 5.9.7,
+# NDK 10e or Qt 5.12.2, NDK 19c, or Qt 6.5.1, NDK 23.1.7779620.
+# COMPILER=cross-android QTPREFIX=/path/to/Qt/6.5.1/android_armv7 \
+#   QTBINARYDIR=/path/to/Qt6.5.1-linux/6.5.1/gcc_64/bin \
+#   ANDROID_SDK_ROOT=/path/to/sdk ANDROID_NDK_ROOT=/path/to/ndk/23.1.7779620 \
+#   ../kid3/build.sh
 #
 # For Linux:
 #
 # To build a self-contained Linux package use
-# COMPILER=gcc-self-contained QTPREFIX=/path/to/Qt5.15.2-linux/5.15.2/gcc_64 ../kid3/build.sh
+# COMPILER=gcc-self-contained QTPREFIX=/path/to/Qt6.5.1-linux/6.5.1/gcc_64 \
+#   ../kid3/build.sh
 #
 # When cross compiling make sure that the host Qt version is not larger than
 # the target Qt version, otherwise moc and plugins will fail. To provide
@@ -182,19 +189,20 @@ fi # makearchive
 # kid3/build.sh rundocker $HOME/projects/kid3/src/build-all.sh
 # You need:
 # - Kid3 project checked out in ~/projects/kid3/src/kid3
+# - At least CMake 3.21 in /opt/cmake/bin/
 # Linux:
-# - Qt 5.15.2 Linux in ~/Development/Qt5.15.2-linux/5.15.2/gcc_64/
+# - Qt 6.5.1 Linux in ~/Development/Qt6.5.1-linux/6.5.1/gcc_64/
 # Windows:
-# - MinGW cross compiler in /opt/mxe/
-# - Qt 5.15.2 MinGW64 in ~/Development/Qt5.15.2-mingw64/5.15.2/mingw81_64/
+# - MinGW 11.2 cross compiler in /opt/mxe11/
+# - Qt 6.5.1 MinGW64 in ~/Development/Qt6.5.1-mingw64/6.5.1/mingw_64/
 # Mac:
 # - Mac cross compiler in /opt/osxcross/
-# - Qt 5.15.2 Mac in ~/Development/Qt5.15.2-mac/5.15.2/clang_64/
+# - Qt 6.5.1 Mac in ~/Development/Qt6.5.1-mac/6.5.1/macos/
 # Android:
-# - Java JDK 8 in /opt/jdk8/
-# - Android SDK in ~/Development/android/sdk/
-# - Android NDK in ~/Development/android/sdk/android-ndk-r19c/
-# - Qt 5.12.12 Android in ~/Development/Qt5.12.12-android/5.12.12/android_armv7/
+# - Java JDK 17
+# - Android SDK in ~/Development/android-sdk/
+# - Android NDK in ~/Development/android-sdk/ndk/23.1.7779620/
+# - Qt 6.5.1 Android in ~/Development/Qt6.5.1-android/6.5.1/android_armv7/
 # - Sign key in ~/Development/ufleisch-release-key.keystore
 # - Gradle cache in ~/.gradle/
 if test "$1" = "makedocker"; then
@@ -205,25 +213,45 @@ if test "$1" = "makedocker"; then
 cd "$(dirname "${BASH_SOURCE[0]}")"
 set -e
 (cd linux_build && \
+   PATH=/opt/cmake/bin:$PATH \
    COMPILER=gcc-self-contained \
-   QTPREFIX=$HOME/Development/Qt5.15.2-linux/5.15.2/gcc_64 \
+   QTPREFIX=$HOME/Development/Qt6.5.1-linux/6.5.1/gcc_64 \
    ../kid3/build.sh)
 (cd mingw64_build && \
-   PATH=/opt/mxe/usr/bin:$PATH \
+   PATH=/opt/mxe11/usr/bin:/opt/cmake/bin:$PATH \
    COMPILER=cross-mingw \
-   QTPREFIX=$HOME/Development/Qt5.15.2-mingw64/5.15.2/mingw81_64 \
+   QTPREFIX=$HOME/Development/Qt6.5.1-mingw64/6.5.1/mingw_64 \
+   QTBINARYDIR=$HOME/Development/Qt6.5.1-linux/6.5.1/gcc_64/bin \
    ../kid3/build.sh)
 (cd macos_build && \
+   rm -f kid3/*-Darwin.dmg && \
+   PATH=/opt/cmake/bin:$PATH \
+   COMPILER=cross-macos \
+   QTPREFIX=$HOME/Development/Qt6.5.1-mac/6.5.1/macos \
+   OSXPREFIX=/opt/osxcross/target \
+   QTBINARYDIR=$HOME/Development/Qt6.5.1-linux/6.5.1/gcc_64/bin \
+   ../kid3/build.sh && \
+   fatdmg=(kid3/*-Darwin.dmg) && \
+   slimdmg=${fatdmg/-Darwin./-Darwin-amd64.} && \
+   PATH=$PATH:/opt/osxcross/target/bin \
+   ../kid3/macosx/mac-strip-arm64.sh $fatdmg $slimdmg)
+(cd macos_qt5_build && \
+   rm -f kid3/*-Darwin.dmg && \
    COMPILER=cross-macos \
    QTPREFIX=$HOME/Development/Qt5.15.2-mac/5.15.2/clang_64 \
    OSXPREFIX=/opt/osxcross/target \
-   ../kid3/build.sh)
+   ../kid3/build.sh && \
+   origdmg=(kid3/*-Darwin.dmg) && \
+   qt5dmg=${origdmg/-Darwin./-Darwin-Qt5.} && \
+   mv $origdmg $qt5dmg)
 (cd android_build && \
+   PATH=/opt/cmake/bin:$PATH \
    COMPILER=cross-android \
-   QTPREFIX=$HOME/Development/Qt5.12.12-android/5.12.12/android_armv7 \
-   JAVA_HOME=/opt/jdk8 \
-   ANDROID_SDK_ROOT=$HOME/Development/android/sdk \
-   ANDROID_NDK_ROOT=$ANDROID_SDK_ROOT/android-ndk-r19c \
+   QTPREFIX=$HOME/Development/Qt6.5.1-android/6.5.1/android_armv7 \
+   QTBINARYDIR=$HOME/Development/Qt6.5.1-linux/6.5.1/gcc_64/bin \
+   JAVA_HOME=/usr/lib/jvm/java-17-openjdk-amd64 \
+   ANDROID_SDK_ROOT=$HOME/Development/android-sdk \
+   ANDROID_NDK_ROOT=$ANDROID_SDK_ROOT/ndk/23.1.7779620 \
    ../kid3/build.sh)
 EOF
     chmod +x build-all.sh
@@ -250,13 +278,15 @@ libvorbis-dev libtag1-dev libchromaprint-dev libavformat-dev \
 libavcodec-dev docbook-xsl pkg-config libreadline-dev xsltproc \
 debian-keyring dput-ng python3-distro-info sudo curl less \
 locales ninja-build ccache p7zip-full genisoimage \
-clang llvm nasm lib32z1 chrpath libpulse-mainloop-glib0
+clang llvm nasm lib32z1 chrpath libpulse-mainloop-glib0 dmg2img archivemount \
+openjdk-17-jre-headless libxcb-cursor0 libxrandr2
 ARG USER
 ARG UID
 RUN adduser --quiet --disabled-password --uid $UID --gecos "User" $USER && \
 echo "$USER:$USER" | chpasswd && usermod -aG sudo $USER && \
 locale-gen en_US.UTF-8 && \
-mkdir -p /home/$USER/projects/kid3 /home/$USER/Development
+mkdir -p /home/$USER/projects/kid3 /home/$USER/Development && \
+ln -s /proc/self/mounts /etc/mtab
 USER $USER
 CMD bash
 EOF
@@ -279,11 +309,13 @@ if test "$1" = "rundocker"; then
     USERNSARG=
   fi
   $DOCKER run $USERNSARG --rm -it -e LANG=C.UTF-8 \
+         --device /dev/fuse --cap-add SYS_ADMIN \
          -v $HOME/projects/kid3:$HOME/projects/kid3 \
          -v $HOME/.gradle:$HOME/.gradle \
          -v $HOME/.gnupg:$HOME/.gnupg:ro \
+         -v /opt/cmake:/opt/cmake:ro \
          -v /opt/osxcross:/opt/osxcross:ro \
-         -v /opt/mxe:/opt/mxe:ro \
+         -v /opt/mxe11:/opt/mxe11:ro \
          -v /opt/jdk8:/opt/jdk8:ro \
          -v $HOME/Development:$HOME/Development:ro ufleisch/kid3dev:bullseye "$@"
   exit 0
@@ -336,7 +368,7 @@ verify_not_in_srcdir
 
 target=${*:-libs package}
 
-qt_version=5.15.2
+qt_version=6.5.1
 zlib_version=1.2.13
 zlib_patchlevel=1
 libogg_version=1.3.4
@@ -345,16 +377,16 @@ libvorbis_version=1.3.7
 libvorbis_patchlevel=1
 ffmpeg3_version=3.2.14
 ffmpeg3_patchlevel=1~deb9u1
-ffmpeg_version=5.1.2
+ffmpeg_version=5.1.3
 ffmpeg_patchlevel=1
 libflac_version=1.4.2+ds
 libflac_patchlevel=2
 id3lib_version=3.8.3
-id3lib_patchlevel=17
-taglib_version=1.13
+id3lib_patchlevel=18
+taglib_version=1.13.1
 chromaprint_version=1.5.1
 chromaprint_patchlevel=2
-mp4v2_version=2.1.1
+mp4v2_version=2.1.3
 
 # Try to find the configuration from an existing build.
 if test -z "$COMPILER"; then
@@ -396,7 +428,9 @@ else
   fi
 fi
 
-if test "$qt_nr" -ge 51204; then
+if test "$compiler" = "cross-android" -o "$compiler" = "gcc-self-contained" && test "$qt_nr" -ge 60000; then
+  openssl_version=3.0.9
+elif test "$qt_nr" -ge 51204; then
   # Since Qt 5.12.4, OpenSSL 1.1.1 is supported
   openssl_version=1.1.1g
 else
@@ -488,6 +522,7 @@ if test $kernel = "MINGW"; then
   CONFIGURE_OPTIONS+=" --prefix=/usr/local"
 elif test $kernel = "Darwin"; then
   CMAKE_OPTIONS="-G \"Unix Makefiles\""
+  _macosx_version_min=10.7
 fi
 
 if test "$compiler" = "cross-mingw"; then
@@ -498,14 +533,19 @@ if test "$compiler" = "cross-mingw"; then
     export CXX=${_crossprefix}g++${_crosssuffix}
   fi
 elif test "$compiler" = "cross-macos"; then
-  CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_TOOLCHAIN_FILE=$thisdir/osxcross.cmake -DCMAKE_C_FLAGS=\"-O2 -mmacosx-version-min=10.7\" -DCMAKE_CXX_FLAGS=\"-O2 -mmacosx-version-min=10.7 -fvisibility=hidden -fvisibility-inlines-hidden -stdlib=libc++\" -DCMAKE_EXE_LINKER_FLAGS=-stdlib=libc++ -DCMAKE_MODULE_LINKER_FLAGS=-stdlib=libc++ -DCMAKE_SHARED_LINKER_FLAGS=-stdlib=libc++"
+  if test "$qt_nr" -ge 60500; then
+    _macosx_version_min=10.15
+  else
+    _macosx_version_min=10.7
+  fi
+  CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_TOOLCHAIN_FILE=$thisdir/osxcross.cmake -DCMAKE_C_FLAGS=\"-O2 -mmacosx-version-min=${_macosx_version_min}\" -DCMAKE_CXX_FLAGS=\"-O2 -mmacosx-version-min=${_macosx_version_min} -fvisibility=hidden -fvisibility-inlines-hidden -stdlib=libc++\" -DCMAKE_EXE_LINKER_FLAGS=-stdlib=libc++ -DCMAKE_MODULE_LINKER_FLAGS=-stdlib=libc++ -DCMAKE_SHARED_LINKER_FLAGS=-stdlib=libc++"
   CONFIGURE_OPTIONS="--host=${cross_host}"
   export CC=${_crossprefix}clang
   export CXX=${_crossprefix}clang++
   export AR=${_crossprefix}ar
-  export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.7"
-  export CXXFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.7 -stdlib=libc++"
-  export LDFLAGS="$ARCH_FLAG -mmacosx-version-min=10.7 -stdlib=libc++"
+  export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=${_macosx_version_min}"
+  export CXXFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=${_macosx_version_min} -stdlib=libc++"
+  export LDFLAGS="$ARCH_FLAG -mmacosx-version-min=${_macosx_version_min} -stdlib=libc++"
 fi
 
 if test $kernel = "MINGW" || test "$compiler" = "cross-mingw"; then
@@ -530,10 +570,10 @@ if test $kernel = "Darwin"; then
     ARCH_FLAG="-Xarch_x86_64"
   fi
   if [[ $(sw_vers -productVersion) = 10.1* || $(sw_vers -productVersion) = 11.* ]]; then
-    CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_C_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.7\" -DCMAKE_CXX_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.7 -fvisibility=hidden -fvisibility-inlines-hidden -stdlib=libc++\" -DCMAKE_EXE_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\" -DCMAKE_MODULE_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\" -DCMAKE_SHARED_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\""
-    export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.7"
-    export CXXFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.7 -stdlib=libc++"
-    export LDFLAGS="$ARCH_FLAG -mmacosx-version-min=10.7 -stdlib=libc++"
+    CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_C_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=${_macosx_version_min}\" -DCMAKE_CXX_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=${_macosx_version_min} -fvisibility=hidden -fvisibility-inlines-hidden -stdlib=libc++\" -DCMAKE_EXE_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\" -DCMAKE_MODULE_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\" -DCMAKE_SHARED_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\""
+    export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=${_macosx_version_min}"
+    export CXXFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=${_macosx_version_min} -stdlib=libc++"
+    export LDFLAGS="$ARCH_FLAG -mmacosx-version-min=${_macosx_version_min} -stdlib=libc++"
   else
     CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_C_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.5\" -DCMAKE_CXX_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.5 -fvisibility=hidden -fvisibility-inlines-hidden\""
     export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.5"
@@ -714,7 +754,7 @@ set(QT_MKSPECS_DIR  \${QT_PREFIX}/mkspecs)
 set(QT_MOC_EXECUTABLE  \${QT_LIBEXEC_DIR}/moc)
 set(QT_UIC_EXECUTABLE  \${QT_LIBEXEC_DIR}/uic)
 
-foreach (_exe moc rcc uic tracegen cmake_automoc_parser qlalr lprodump lrelease-pro lupdate-pro)
+foreach (_exe moc rcc uic tracegen cmake_automoc_parser qlalr lprodump lrelease-pro lupdate-pro tracepointgen)
   if (NOT TARGET Qt${qt_version_major}::\${_exe})
     add_executable(Qt${qt_version_major}::\${_exe} IMPORTED)
     set_target_properties(Qt${qt_version_major}::\${_exe} PROPERTIES
@@ -722,7 +762,7 @@ foreach (_exe moc rcc uic tracegen cmake_automoc_parser qlalr lprodump lrelease-
     )
   endif ()
 endforeach (_exe)
-foreach (_exe lupdate lrelease qtpaths androiddeployqt androidtestrunner lconvert)
+foreach (_exe lupdate lrelease windeployqt qtpaths androiddeployqt androidtestrunner lconvert)
   if (NOT TARGET Qt${qt_version_major}::\${_exe})
     add_executable(Qt${qt_version_major}::\${_exe} IMPORTED)
     set_target_properties(Qt${qt_version_major}::\${_exe} PROPERTIES
@@ -770,15 +810,17 @@ endif (POLICY CMP0025)
 
 set(QT_PREFIX $_qt_prefix)
 
-set (CMAKE_SYSTEM_NAME Darwin)
-set (CMAKE_C_COMPILER $osxprefix/lib/ccache/bin/${_crossprefix}clang)
-set (CMAKE_CXX_COMPILER $osxprefix/lib/ccache/bin/${_crossprefix}clang++)
-set (CMAKE_FIND_ROOT_PATH $osxprefix/${cross_host};$osxsdk/usr;$osxprefix/${cross_host};$osxsdk;$osxsdk/System/Library/Frameworks/OpenGL.framework/Versions/A/Headers;\${QT_PREFIX};$thisdir/buildroot/usr/local)
-set (CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
-set (CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
-set (CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
-set (CMAKE_AR:FILEPATH ${_crossprefix}ar)
-set (CMAKE_RANLIB:FILEPATH ${_crossprefix}ranlib)
+set(CMAKE_SYSTEM_NAME Darwin)
+set(CMAKE_C_COMPILER $osxprefix/lib/ccache/bin/${_crossprefix}clang)
+set(CMAKE_CXX_COMPILER $osxprefix/lib/ccache/bin/${_crossprefix}clang++)
+set(CMAKE_FIND_ROOT_PATH $osxprefix/${cross_host};$osxsdk/usr;$osxprefix/${cross_host};$osxsdk;$osxsdk/System/Library/Frameworks/OpenGL.framework/Versions/A/Headers;\${QT_PREFIX};$thisdir/buildroot/usr/local)
+set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
+set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
+set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
+set(CMAKE_AR:FILEPATH ${_crossprefix}ar)
+set(CMAKE_RANLIB:FILEPATH ${_crossprefix}ranlib)
+set(CMAKE_SHARED_LIBRARY_RUNTIME_C_FLAG "-Wl,-rpath,")
+set(CMAKE_SHARED_LIBRARY_RUNTIME_C_FLAG_SEP ":")
 
 EOF
     if test "$qt_version_major" = "6"; then
@@ -793,7 +835,7 @@ set(QT_MKSPECS_DIR  \${QT_PREFIX}/mkspecs)
 set(QT_MOC_EXECUTABLE  \${QT_LIBEXEC_DIR}/moc)
 set(QT_UIC_EXECUTABLE  \${QT_LIBEXEC_DIR}/uic)
 
-foreach (_exe moc rcc uic tracegen cmake_automoc_parser qlalr lprodump lrelease-pro lupdate-pro)
+foreach (_exe moc rcc uic tracegen cmake_automoc_parser qlalr lprodump lrelease-pro lupdate-pro tracepointgen)
   if (NOT TARGET Qt${qt_version_major}::\${_exe})
     add_executable(Qt${qt_version_major}::\${_exe} IMPORTED)
     set_target_properties(Qt${qt_version_major}::\${_exe} PROPERTIES
@@ -801,7 +843,7 @@ foreach (_exe moc rcc uic tracegen cmake_automoc_parser qlalr lprodump lrelease-
     )
   endif ()
 endforeach (_exe)
-foreach (_exe lupdate lrelease qtpaths androiddeployqt androidtestrunner lconvert)
+foreach (_exe lupdate lrelease macdeployqt qtpaths androiddeployqt androidtestrunner lconvert)
   if (NOT TARGET Qt${qt_version_major}::\${_exe})
     add_executable(Qt${qt_version_major}::\${_exe} IMPORTED)
     set_target_properties(Qt${qt_version_major}::\${_exe} PROPERTIES
@@ -969,15 +1011,17 @@ if test "$compiler" = "cross-android"; then
     echo "### Extracting openssl"
 
     tar xzf source/openssl-${openssl_version}.tar.gz
-    cp source/Setenv-android.sh openssl-${openssl_version}/
-    cd openssl-${openssl_version}/
-    sed -i 's/\r$//' Setenv-android.sh
-    test -n "$ANDROID_NDK_ROOT" && \
-      sed -i "s#^ANDROID_NDK_ROOT=.*#ANDROID_NDK_ROOT=$ANDROID_NDK_ROOT#" \
-        Setenv-android.sh
-    chmod +x Setenv-android.sh
-    patch -p0 <$srcdir/packaging/patches/openssl-1.1.1-android00-setenv.patch
-    cd ..
+    if test "$qt_nr" -lt 60000; then
+      cp source/Setenv-android.sh openssl-${openssl_version}/
+      cd openssl-${openssl_version}/
+      sed -i 's/\r$//' Setenv-android.sh
+      test -n "$ANDROID_NDK_ROOT" && \
+        sed -i "s#^ANDROID_NDK_ROOT=.*#ANDROID_NDK_ROOT=$ANDROID_NDK_ROOT#" \
+          Setenv-android.sh
+      chmod +x Setenv-android.sh
+      patch -p0 <$srcdir/packaging/patches/openssl-1.1.1-android00-setenv.patch
+      cd ..
+    fi
   fi
 
 elif test "$compiler" = "gcc-self-contained" || test "$compiler" = "gcc-debug" \
@@ -1027,36 +1071,45 @@ if test "$compiler" = "cross-android"; then
     echo "### Building OpenSSL"
 
     cd openssl-${openssl_version}/
-    if test "$_android_abi" = "x86"; then
-      sed -i 's/^_ANDROID_EABI=.*$/_ANDROID_EABI=x86-4.9/; s/^_ANDROID_ARCH=.*$/_ANDROID_ARCH=arch-x86/' Setenv-android.sh
-    else
-      sed -i 's/^_ANDROID_EABI=.*$/_ANDROID_EABI=arm-linux-androideabi-4.9/; s/^_ANDROID_ARCH=.*$/_ANDROID_ARCH=arch-arm/' Setenv-android.sh
-    fi
-    sed -i "s#^_ANDROID_NDK=.*#ANDROID_NDK_ROOT=$_android_ndk_root#" Setenv-android.sh
-    sed -i '/FIPS_SIG location/,/^fi$/ d' Setenv-android.sh
-    if test -d $_android_ndk_root/toolchains/llvm; then
-      sed -i 's/^_ANDROID_EABI=.*$/_ANDROID_EABI=llvm/' Setenv-android.sh
-    fi
-    . ./Setenv-android.sh
-    if test "${openssl_version:0:3}" = "1.0"; then
-      ./Configure shared android
-    else
-      ANDROID_NDK_HOME=$ANDROID_NDK_ROOT ./Configure shared android-armeabi
-    fi
-    if test -d $_android_ndk_root/toolchains/llvm; then
+    if test "$qt_nr" -lt 60000; then
       if test "$_android_abi" = "x86"; then
-        sed -i 's/^CC=.*$/CC= i686-linux-android16-clang/; s/ -mandroid//' Makefile
+        sed -i 's/^_ANDROID_EABI=.*$/_ANDROID_EABI=x86-4.9/; s/^_ANDROID_ARCH=.*$/_ANDROID_ARCH=arch-x86/' Setenv-android.sh
       else
-        sed -i 's/^CC=.*$/CC= armv7a-linux-androideabi16-clang/; s/ -mandroid//' Makefile
+        sed -i 's/^_ANDROID_EABI=.*$/_ANDROID_EABI=arm-linux-androideabi-4.9/; s/^_ANDROID_ARCH=.*$/_ANDROID_ARCH=arch-arm/' Setenv-android.sh
+      fi
+      sed -i "s#^_ANDROID_NDK=.*#ANDROID_NDK_ROOT=$_android_ndk_root#" Setenv-android.sh
+      sed -i '/FIPS_SIG location/,/^fi$/ d' Setenv-android.sh
+      if test -d $_android_ndk_root/toolchains/llvm; then
+        sed -i 's/^_ANDROID_EABI=.*$/_ANDROID_EABI=llvm/' Setenv-android.sh
+      fi
+      . ./Setenv-android.sh
+      if test "${openssl_version:0:3}" = "1.0"; then
+        ./Configure shared android
+      else
+        ANDROID_NDK_HOME=$ANDROID_NDK_ROOT ./Configure shared android-armeabi
+      fi
+      if test -d $_android_ndk_root/toolchains/llvm; then
+        if test "$_android_abi" = "x86"; then
+          sed -i 's/^CC=.*$/CC= i686-linux-android16-clang/; s/ -mandroid//' Makefile
+        else
+          sed -i 's/^CC=.*$/CC= armv7a-linux-androideabi16-clang/; s/ -mandroid//' Makefile
+        fi
       fi
-    fi
 
-    if test "${openssl_version:0:3}" = "1.0"; then
-      make CALC_VERSIONS="SHLIB_COMPAT=; SHLIB_SOVER=" build_libs
-      _ssl_lib_suffix=.so
+      if test "${openssl_version:0:3}" = "1.0"; then
+        make CALC_VERSIONS="SHLIB_COMPAT=; SHLIB_SOVER=" build_libs
+        _ssl_lib_suffix=.so
+      else
+        make ANDROID_NDK_HOME=$ANDROID_NDK_ROOT SHLIB_VERSION_NUMBER= SHLIB_EXT=_1_1.so build_libs
+        _ssl_lib_suffix=_1_1.so
+      fi
     else
-      make ANDROID_NDK_HOME=$ANDROID_NDK_ROOT SHLIB_VERSION_NUMBER= SHLIB_EXT=_1_1.so build_libs
-      _ssl_lib_suffix=_1_1.so
+      PATH=$ANDROID_NDK_ROOT/toolchains/llvm/prebuilt/linux-x86_64/bin:$PATH
+      sed -i 's/shared_extension => ".so",/shared_extension => "_3.so",/' Configurations/15-android.conf
+      ANDROID_NDK_HOME=$ANDROID_NDK_ROOT ./Configure shared android-arm
+      make ANDROID_NDK_HOME=$ANDROID_NDK_ROOT SHLIB_VERSION_NUMBER= SHLIB_EXT=_3.so build_libs
+      _ssl_lib_suffix=_3.so
+      _android_prefix=llvm
     fi
     mkdir -p inst/usr/local/lib
     cp --dereference libssl${_ssl_lib_suffix} libcrypto${_ssl_lib_suffix} inst/usr/local/lib/
@@ -1197,6 +1250,9 @@ else #  cross-android
     configure_args="--enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local --disable-thorough-tests --disable-doxygen-docs --disable-xmms-plugin $FLAC_BUILD_OPTION $CONFIGURE_OPTIONS"
     if test $kernel = "Darwin"; then
       configure_args="$configure_args --disable-asm-optimizations"
+    elif test $kernel = "MINGW" || test "$compiler" = "cross-mingw"; then
+      # Avoid having to ship with libssp-0.dll
+      configure_args="$configure_args --disable-stack-smash-protection --disable-fortify-source"
     fi
     test -f Makefile || ./configure $configure_args
     # On msys32, an error "changed before entering" occurred, can be fixed by
@@ -1442,7 +1498,7 @@ _android_keystore_alias=
 fi
 _buildprefix=\$(cd ..; pwd)/buildroot/usr/local
 # Pass -DQT_ANDROID_USE_GRADLE=ON to use Gradle instead of ANT.
-cmake -DJAVA_HOME=\$_java_root -DQT_ANDROID_SDK_ROOT=\$_android_sdk_root -DANDROID_NDK=\$_android_ndk_root -DAPK_ALL_TARGET=OFF -DANDROID_ABI=\$_android_abi -DANDROID_EXTRA_LIBS_DIR=\$_buildprefix/lib -DANDROID_KEYSTORE_PATH=\$_android_keystore_path -DANDROID_KEYSTORE_ALIAS=\$_android_keystore_alias -DCMAKE_TOOLCHAIN_FILE=$_android_toolchain_cmake -DANDROID_PLATFORM=$_android_platform -DANDROID_CCACHE=$_android_ccache -DQT_QMAKE_EXECUTABLE=\$_android_qt_root/bin/qmake -DCMAKE_BUILD_TYPE=Release -DDOCBOOK_XSL_DIR=${_docbook_xsl_dir} -DPYTHON_EXECUTABLE=/usr/bin/python -DXSLTPROC=/usr/bin/xsltproc -DGZIP_EXECUTABLE=/bin/gzip -DTAGLIBCONFIG_EXECUTABLE=\$_buildprefix/bin/taglib-config -DCMAKE_MAKE_PROGRAM=make $srcdir
+cmake -DJAVA_HOME=\$_java_root -DQT_ANDROID_SDK_ROOT=\$_android_sdk_root -DANDROID_SDK_ROOT=\$_android_sdk_root -DANDROID_NDK=\$_android_ndk_root -DAPK_ALL_TARGET=OFF -DANDROID_ABI=\$_android_abi -DANDROID_EXTRA_LIBS_DIR=\$_buildprefix/lib -DANDROID_KEYSTORE_PATH=\$_android_keystore_path -DANDROID_KEYSTORE_ALIAS=\$_android_keystore_alias -DCMAKE_TOOLCHAIN_FILE=$_android_toolchain_cmake -DANDROID_PLATFORM=$_android_platform -DANDROID_CCACHE=$_android_ccache -DQT_QMAKE_EXECUTABLE=\$_android_qt_root/bin/qmake -DQT_HOST_PATH=${QTBINARYDIR%/bin} -DCMAKE_BUILD_TYPE=Release -DDOCBOOK_XSL_DIR=${_docbook_xsl_dir} -DPYTHON_EXECUTABLE=/usr/bin/python -DXSLTPROC=/usr/bin/xsltproc -DGZIP_EXECUTABLE=/bin/gzip -DTAGLIBCONFIG_EXECUTABLE=\$_buildprefix/bin/taglib-config -DCMAKE_MAKE_PROGRAM=make $srcdir
 EOF
       chmod +x kid3/run-cmake.sh
     fi
@@ -1531,7 +1587,7 @@ EOF
 #!/bin/bash
 BUILDPREFIX=\$(cd ..; pwd)/buildroot/usr/local
 export PKG_CONFIG_PATH=\$BUILDPREFIX/lib/pkgconfig
-cmake -GNinja -DBUILD_SHARED_LIBS=ON -DQT_QMAKE_EXECUTABLE=${QT_PREFIX}/bin/qmake -DLINUX_SELF_CONTAINED=ON -DWITH_READLINE=OFF -DWITH_TAGLIB=OFF -DHAVE_TAGLIB=1 -DTAGLIB_LIBRARIES:STRING="-L\$BUILDPREFIX/lib -ltag -lz" -DTAGLIB_CFLAGS:STRING="-I\$BUILDPREFIX/include/taglib -I\$BUILDPREFIX/include -DTAGLIB_STATIC" -DTAGLIB_VERSION:STRING="${taglib_config_version}" -DWITH_QML=ON -DCMAKE_CXX_FLAGS_DEBUG:STRING="-g -DID3LIB_LINKOPTION=1 -DFLAC__NO_DLL" -DCMAKE_INCLUDE_PATH=\$BUILDPREFIX/include -DCMAKE_LIBRARY_PATH=\$BUILDPREFIX/lib -DCMAKE_PROGRAM_PATH=\$BUILDPREFIX/bin -DWITH_FFMPEG=ON -DFFMPEG_ROOT=\$BUILDPREFIX -DWITH_MP4V2=ON $CMAKE_BUILD_OPTION -DWITH_APPS="Qt;CLI" -DCMAKE_INSTALL_PREFIX= -DWITH_BINDIR=. -DWITH_DATAROOTDIR=. -DWITH_DOCDIR=. -DWITH_TRANSLATIONSDIR=. -DWITH_LIBDIR=. -DWITH_PLUGINSDIR=./plugins ../../kid3
+cmake -GNinja -DBUILD_SHARED_LIBS=ON -DQT_QMAKE_EXECUTABLE=${QTPREFIX:-$QT_PREFIX}/bin/qmake -DLINUX_SELF_CONTAINED=ON -DWITH_READLINE=OFF -DWITH_TAGLIB=OFF -DHAVE_TAGLIB=1 -DTAGLIB_LIBRARIES:STRING="-L\$BUILDPREFIX/lib -ltag -lz" -DTAGLIB_CFLAGS:STRING="-I\$BUILDPREFIX/include/taglib -I\$BUILDPREFIX/include -DTAGLIB_STATIC" -DTAGLIB_VERSION:STRING="${taglib_config_version}" -DWITH_QML=ON -DCMAKE_CXX_FLAGS_DEBUG:STRING="-g -DID3LIB_LINKOPTION=1 -DFLAC__NO_DLL" -DCMAKE_INCLUDE_PATH=\$BUILDPREFIX/include -DCMAKE_LIBRARY_PATH=\$BUILDPREFIX/lib -DCMAKE_PROGRAM_PATH=\$BUILDPREFIX/bin -DWITH_FFMPEG=ON -DFFMPEG_ROOT=\$BUILDPREFIX -DWITH_MP4V2=ON $CMAKE_BUILD_OPTION -DWITH_APPS="Qt;CLI" -DCMAKE_INSTALL_PREFIX= -DWITH_BINDIR=. -DWITH_DATAROOTDIR=. -DWITH_DOCDIR=. -DWITH_TRANSLATIONSDIR=. -DWITH_LIBDIR=. -DWITH_PLUGINSDIR=./plugins ../../kid3
 EOF
       else
         taglib_config_version=$taglib_version
@@ -1604,9 +1660,12 @@ EOF
     dmg dmg uncompressed.dmg kid3-$_version-Darwin.dmg
     rm uncompressed.dmg
   elif test "$compiler" = "cross-android"; then
-    JAVA_HOME=$(grep _java_root= run-cmake.sh | cut -d'=' -f2) make apk
+    JAVA_HOME=$(grep _java_root= run-cmake.sh | cut -d'=' -f2) \
+    QT_ANDROID_KEYSTORE_PATH=$(grep ANDROID_KEYSTORE_PATH CMakeCache.txt | cut -d= -f2) \
+    QT_ANDROID_KEYSTORE_ALIAS=$(grep ANDROID_KEYSTORE_ALIAS CMakeCache.txt | cut -d= -f2) \
+    make apk
     _version=$(grep VERSION config.h | cut -d'"' -f2)
-    for prefix in android/build/outputs/apk/release/android-release android/build/outputs/apk/android-release android/bin/QtApp-release; do
+    for prefix in android/build/outputs/apk/release/android-release android/build/outputs/apk/android-release android/bin/QtApp-release android/android-build/build/outputs/apk/release/android-build-release android/android-build/build/outputs/apk/debug/android-build-debug; do
       for suffix in signed unsigned; do
         _apkpath=${prefix}-${suffix}.apk
         if test -f $_apkpath; then
