@@ -34,6 +34,9 @@
 #endif
 #include <QQmlApplicationEngine>
 #include <QQuickStyle>
+#if QT_VERSION > 0x060200 && defined Q_OS_ANDROID
+#include <QtCore/private/qandroidextras_p.h>
+#endif
 #include <typeinfo>
 #include "config.h"
 #include "loadtranslation.h"
@@ -115,8 +118,21 @@ int main(int argc, char* argv[])
   Q_INIT_RESOURCE(translations);
 #endif
 
+#if QT_VERSION > 0x060200 && defined Q_OS_ANDROID
+  const QString storagePermission =
+      QLatin1String("android.permission.WRITE_EXTERNAL_STORAGE");
+  auto permissionResult =
+      QtAndroidPrivate::checkPermission(storagePermission).result();
+  if (permissionResult != QtAndroidPrivate::Authorized) {
+    permissionResult =
+        QtAndroidPrivate::requestPermission(storagePermission).result();
+  }
+#endif
+
   QCoreApplication::setApplicationName(QLatin1String("Kid3"));
+#if QT_VERSION < 0x060000
   QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
 
   // The QtQuickStyle setting has to be read bypassing the regular
   // configuration object because the style environment variable
